// // ForGet. A tool for managing development dependencies.
// // Copyright (C) 2015  Daniel Hughes
// //
// // Permission is hereby granted, free of charge, to any person obtaining a copy
// // of this software and associated documentation files (the "Software"), to deal
// // in the Software without restriction, including without limitation the rights
// // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// // copies of the Software, and to permit persons to whom the Software is
// // furnished to do so, subject to the following conditions:
// //
// // The above copyright notice and this permission notice shall be included in
// // all copies or substantial portions of the Software.
// //
// // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// // AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// // THE SOFTWARE.
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace ForGet
{
   public class Archive
   {
      public bool Install(string source, string destination)
      {
         _messageBuilder.Clear();
         using(var process = new Process())
         {
            process.StartInfo = new ProcessStartInfo ()
            {
               FileName = "7z",
               Arguments = string.Format(@"x ""{0}"" -o""{1}""", source, destination),
               RedirectStandardError = true,
               RedirectStandardOutput = true,
               UseShellExecute = false,
            };
            
            process.OutputDataReceived += ProcessDataReceived;
            process.ErrorDataReceived += ProcessDataReceived;
            if(!process.Start())
            {
               Console.Error.WriteLine("ERROR: Failed to start zip tool");
               return false;
            }
            process.BeginOutputReadLine();
            process.BeginErrorReadLine();
            process.WaitForExit();
            if(process.ExitCode != 0)
            {
               Console.Error.WriteLine("ERROR: 7z.exe failed to unzip package:");
               Console.Error.WriteLine(_messageBuilder.ToString());
               return false;
            }

            FixUrlEncodedPaths(destination);

            return true;
         }
      }

      void FixUrlEncodedPaths(string path)
      {
         foreach (var folder in Directory.GetDirectories(path, "*", SearchOption.AllDirectories))
         {
            string decoded = System.Web.HttpUtility.UrlDecode(folder);
            if (decoded != folder && Directory.Exists(folder))
            {
               Directory.Move(folder, decoded);
            }
         }
         foreach (var file in Directory.GetFiles(path, "*", SearchOption.AllDirectories))
         {
            string decoded = System.Web.HttpUtility.UrlDecode(file);
            if (decoded != file)
            {
               File.Move(file, decoded);
            }
         }
      }

      StringBuilder _messageBuilder = new StringBuilder ();

      void ProcessDataReceived(object sender, DataReceivedEventArgs e)
      {
         if(e.Data != null)
         {
            if(e.Data.Length != 0)
            {
               _messageBuilder.Append(e.Data);
               _messageBuilder.AppendLine();
            }
         }
      }
   }
}

