// ForGet. A tool for managing development dependencies.
// Copyright (C) 2015  Daniel Hughes
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using NDesk.Options;

namespace ForGet
{
   class MainClass
   {
      const string _configFile = "dependencies.config";
    
      public static int Main(string[] args)
      {
         bool restore = false;
         bool upgrade = false;
         bool create = false;
         bool showHelp = false;
         var options = new OptionSet () 
         {
            { "r|restore", "Pull down all dependencies as specified in the dependencies.config file.", v => restore = v != null },
            { "u|upgrade", "Upgrades the versions in the dependency.config file based on the UpgradeType",  v => upgrade = v != null},
            { "c|create", "Creates a template dependencies.config ",  v => create = v != null},
            { "h|help",  "Show this message and exit", v => showHelp = v != null },
         };
       
         string dependenciesConfigPath = _configFile;
         string basePath = Environment.CurrentDirectory;

         try
         {
            var extra = options.Parse(args);
            if(extra.Count > 0)
            {
               dependenciesConfigPath = extra[0];
               basePath = System.IO.Path.GetDirectoryName(dependenciesConfigPath);
            }
         }
         catch(OptionException exception)
         {
            Console.Write("ForGet: ");
            Console.Error.WriteLine(exception.Message);
            Console.WriteLine("Try `ForGet --help' for more information.");
            return - 1;
         }
       
         if(showHelp)
         {
            ShowHelp(options);
            return 0;
         }
         if(upgrade)
         {
            if(!new Upgrader ().Upgrade(_configFile))
            {
               return -1;
            }
         }
         if(restore)
         {
            if(!new Restorer (basePath).Restore(dependenciesConfigPath))
            {
               return -1;
            }
         }
         if(create)
         {
            if(new Creator ().Create(_configFile))
            {
               return -1;  
            }
         }
         return 0;
      }
    
      public static void ShowHelp(OptionSet optionsSet)
      {
         Console.WriteLine("Usage: ForGet [OPTIONS]+ message");
         Console.WriteLine("A tool for managing development dependencies.");
         Console.WriteLine();
         Console.WriteLine("Options:");
         optionsSet.WriteOptionDescriptions(Console.Out);
      }
   }
}
